package net.smart.core;

import java.util.*;

public class SmartCoreTransformation
{
	public final String className;
	public final String methodName;
	public final String[] parameterTypeNames;
	public final boolean[] isHookParameter;
	public final String returnType;
	public final String hookClassName;
	public final String hookName;
	public final boolean addBeforeHook;
	public final boolean addAfterHook;
	public final boolean addThisAsFirstParameter;

	public SmartCoreTransformation(String className, String methodName, String[] parameterTypeNames, boolean[] isHookParameter, String returnType, String hookClassName, String hookName)
	{
		this(className, methodName, parameterTypeNames, isHookParameter, returnType, hookClassName, hookName, true, true, true);
	}

	public SmartCoreTransformation(String className, String methodName, String[] parameterTypeNames, boolean[] isHookParameter, String returnType, String hookClassName, String hookName, boolean addBeforeHook, boolean addAfterHook, boolean addThisAsFirstParameter)
	{
		this.className = className;
		this.methodName = methodName;
		this.parameterTypeNames = parameterTypeNames;
		this.isHookParameter = isHookParameter;
		this.returnType = returnType;
		this.hookClassName = hookClassName;
		this.hookName = hookName;
		this.addBeforeHook = addBeforeHook;
		this.addAfterHook = addAfterHook;
		this.addThisAsFirstParameter = addThisAsFirstParameter;
	}

	private static String getDescription(String type)
	{
		if(type == null)
			return "V";
		if(type.length() == 1 && Character.isUpperCase(type.charAt(0)))
			return type;
		return "L" + type.replace(".", "/") + ";";
	}

	private static String getDescriptions(String[] parameterTypes)
	{
		String result = "";
		for(int i=0; i<parameterTypes.length; i++)
			result += getDescription(parameterTypes[i]);
		return result + "";
	}

	private static String getMethodDescription(String[] parameterTypes, String resultType)
	{
		return "(" + getDescriptions(parameterTypes) + ")" + getDescription(resultType);
	}

	public String getMethodDesc()
	{
		return getMethodDescription(parameterTypeNames, returnType);
	}

	public String getHookClassDesc()
	{
		return hookClassName.replace(".", "/");
	}

	public String getHookName(String prefix)
	{
		String[] className = this.className.split("\\.");
		String name = hookName != null ? hookName : methodName;
		return className[className.length-1] + "_" + prefix + Character.toUpperCase(name.charAt(0)) + name.substring(1);
	}

	public String getHookMethodDesc()
	{
		List names = new ArrayList();
		if(addThisAsFirstParameter)
			names.add(className);
		for(int i=0; parameterTypeNames != null && i<parameterTypeNames.length; i++)
			if(isHookParameter(i))
				names.add(parameterTypeNames[i]);

		String[] result = new String[names.size()];
		names.toArray(result);
		return getMethodDescription(result, null);
	}

	public boolean isHookParameter(int i)
	{
		return isHookParameter != null && isHookParameter.length > i && isHookParameter[i];
	}
}